/**
 * Copyright 2010 The Regents of the University of California
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on
 * an "AS IS"; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under
 * the License.
 * 
 * Author: Rares Vernica <rares (at) ics.uci.edu>
 */

package edu.uci.ics.fuzzyjoin.recordgroup;

import java.util.Collection;
import java.util.LinkedList;

import edu.uci.ics.fuzzyjoin.similarity.SimilarityFilters;

public class RecordGroupTokenFrequencyMirror extends RecordGroup {

    private final Collection<Integer> groups = new LinkedList<Integer>();

    public RecordGroupTokenFrequencyMirror(int noGroups,
            SimilarityFilters fuzzyFilters) {
        super(noGroups, fuzzyFilters);
    }

    @Override
    public Iterable<Integer> getGroups(Integer token, Integer length) {
        int noGroupsDouble = noGroups << 1;
        int group = token % noGroupsDouble;
        if (group >= noGroups) {
            // mirror
            group = -(group - noGroupsDouble) - 1;
        }
        groups.clear();
        groups.add(group);
        return groups;
    }

    @Override
    public boolean isLengthOnly() {
        return false;
    }

}
